// LED ON/OFF Toggle with the Clap Sounds 

// With Zero-crossing rate and own algorithm 

#include <DHT11.h> 

#include <FastLED.h> 

 

#define DHTPIN 2       

DHT11 dht11(DHTPIN); 

#define micPin A0  // Sound Sensor Connection at A0 Analog pin 

#define LED_PIN1 8 // LED PIN for Thunder Part 

#define LED_PIN2 9 // LED PIN for Cloud Part 

#define NUM_LEDS1 1 // Number of LEDs for Thunder Part 

#define NUM_LEDS2 12 // Number of LEDs for Cloud Part 

 

CRGB led1[NUM_LEDS1]; // Declare led1 for Thunder Part 

CRGB led2[NUM_LEDS2]; // Declare led2 for Cloud Part 

 

bool isLightOn = false; 

 

void clap_flag(){ 

  int amp, prev_amp; 

  prev_amp = analogRead(micPin) - 512; 

  #define sign(x) ( (x>=0) ? +1 : -1 ) 

     

  int zeroCrossings = 0;   

  double tmp100 = 0; 

  double tmp200 = 0; 

  double tmp300 = 0; 

  double tmp400 = 0; 

  double tmp500 = 0; 

 

  for (int i = 0; i <= 2000; i++) { 

    amp = analogRead(micPin) - 512; 

    if(abs(amp) > 100)tmp100++; 

    if(abs(amp) > 200)tmp200++;   

    if(abs(amp) > 300)tmp300++; 

    if(abs(amp) > 400)tmp400++; 

    if(abs(amp) > 500)tmp500++; 

    if (sign(amp) != sign(prev_amp)) zeroCrossings++; 

    prev_amp = amp; 

  } 

 

  if(tmp100 == 0)return; 

  if(tmp200 == 0)return; 

  if(tmp300 == 0)return; 

  if(tmp400 == 0)return; 

  if(tmp500 == 0)return; 

  if ((zeroCrossings > 700)) {  

    Serial.print("zeroCrossings : "); // For Debugging 

    Serial.println(zeroCrossings); 

    Serial.print("100 : ");   //782, 837, 837, 762, 742 

    Serial.println(tmp100);    

    Serial.print("200 : ");   //592, 688, 700, 595, 575 

    Serial.println(tmp200); 

    Serial.print("300 : ");   //171, 211, 224, 363, 190 

    Serial.println(tmp300); 

    Serial.print("400 : ");   //171, 211, 224, 363, 190 

    Serial.println(tmp400); 

    Serial.print("500 : ");   //171, 211, 224, 363, 190 

    Serial.println(tmp500); 

 

    if(!(3 > tmp100 / tmp200 && tmp100 / tmp200 > 1.4))return; 

    if(!(4.2> tmp200 / tmp300 && tmp200 / tmp300 > 3.0))return; 

    if(!(2 >tmp300 / tmp400 && tmp300 / tmp400 > 1.0))return; 

    if(!(2 > tmp400 / tmp500 && tmp400 / tmp500 > 1.0))return; 

    isLightOn = !isLightOn; 

  } 

} 

 

void light_flag() { 

  if (isLightOn) { 

    led2[0] = CRGB(0, 50, 0); // Turn ON the LED(Cloud Part) 

    FastLED.show(); 

  } else { 

    led2[0] = CRGB(0, 0, 0); // Turn OFF the LED(Cloud Part) 

    FastLED.show(); 

  } 

} 

 

void setup() { 

  FastLED.addLeds<WS2812, LED_PIN1, GRB>(led1, NUM_LEDS1); 

  FastLED.addLeds<NEOPIXEL, LED_PIN2>(led2, NUM_LEDS2); 

  Serial.begin(9600); 

  FastLED.clear(); // Set the Initial State the LED 

  FastLED.show(); 

} 

 

void loop() { 

  led1[0] = CRGB(255, 255, 0); // Always Turn ON the LED(Thunder Part) 

  double soundLevel = analogRead(micPin); // analog value from sound sensor 

  if(soundLevel > 600 || soundLevel < 400){ 

    clap_flag(); 

  } 

  light_flag(); 

} 

 

 